/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.common;


/**
 * This class allows to run a command in sub-process.
 */


import java.io.IOException;
import java.lang.Process;
import java.lang.String;


public class Cmd {

    private String mCmd;

    /**
     * Construtor method of Cmd class.
     * @param cmd command
     */
    public Cmd(String cmd) {
	mCmd = new String(cmd);
    }

    /**
     * Execute the command.
     * (In case of the exit value of the command is not 0,
     * the error stream of the command is traced.)
     * @return false if the command cannot be executed, or if its exit value is not 0.
     */
    public boolean run() {
	boolean val = true;
	try {
	    Process p = Runtime.getRuntime().exec(mCmd);
	    p.waitFor();
	    if (p.exitValue() != 0) {
		Trace.errln(p.getErrorStream());
		val = false;
	    }
	} catch (Exception e) {
	    Trace.errln(e, true);
	    val = false;
	}
	return(val);
    }
}

