/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.lib;


import javax.ejb.deployment.ControlDescriptor;
import javax.ejb.deployment.SessionDescriptor;
import org.objectweb.jonas.common.Trace;


/**
 * This class implements some useful functions for converting the values of
 * transaction attributes, isolation level attributes, ....
 * from int value to string value and vice versa for trace need.
 */
public class Util {

    public final static int TX_ATTR_NOTDEFINED          = -1998;
    public final static int TX_ISOLEVEL_NOTDEFINED      = -1998;
    public final static int RUNASMODE_NOTDEFINED        = -1998;
    public final static int SESSIONSTATETYPE_NOTDEFINED = -1998;

    /**
     * Gets the int value of a transaction attribute from a transaction attribute string.
     * Used for tracing purpose.
     *
     * @param  String attributeName The Transaction Attribute Name
     * @return int    attr          The Transaction Attribute
     */
    public static int getTransactionAttribute (String attributeName) {

	if (attributeName.equals("TX_BEAN_MANAGED"))  return ControlDescriptor.TX_BEAN_MANAGED;
	if (attributeName.equals("TX_MANDATORY"))     return ControlDescriptor.TX_MANDATORY;
	if (attributeName.equals("TX_NOT_SUPPORTED")) return ControlDescriptor.TX_NOT_SUPPORTED;
	if (attributeName.equals("TX_REQUIRED"))      return ControlDescriptor.TX_REQUIRED;
	if (attributeName.equals("TX_REQUIRES_NEW"))  return ControlDescriptor.TX_REQUIRES_NEW;
	if (attributeName.equals("TX_SUPPORTS"))      return ControlDescriptor.TX_SUPPORTS;
	return TX_ATTR_NOTDEFINED;
    }

 

    /**
     * Gets the string value of an transaction from a transaction attribute int.
     * Used for tracing purpose.
     *
     * @param  int    attr          The Transaction Attribute
     * @return String attributeName The Transaction Attribute Name
     */
    public static String getTransactionAttributeName (int attr) {

	  switch (attr) {
	  case TX_ATTR_NOTDEFINED:        return("TX_ATTR_NOTDEFINED");
	  case ControlDescriptor.TX_BEAN_MANAGED:  return("TX_BEAN_MANAGED");
	  case ControlDescriptor.TX_MANDATORY:     return("TX_MANDATORY");
	  case ControlDescriptor.TX_NOT_SUPPORTED: return("TX_NOT_SUPPORTED");
	  case ControlDescriptor.TX_REQUIRED:      return("TX_REQUIRED");
	  case ControlDescriptor.TX_REQUIRES_NEW:  return("TX_REQUIRES_NEW");
	  case ControlDescriptor.TX_SUPPORTS:      return("TX_SUPPORTS");
	  default:                                 return("TX_ATTR_NOTDEFINED");
	  }
      }

    /**
     * Gets the int value of a transaction isolation level from a transaction isolation
     * level string.
     * Used for tracing purpose.
     *
     * @param  String isoLevelName  The Transaction Isolation Level Name
     * @return int                  The Transaction Isolation Level
     */
    public static int getTransactionIsolationLevel (String isoLevelName) {

	if (isoLevelName.equals("TRANSACTION_READ_UNCOMMITTED")) {
	    return(ControlDescriptor.TRANSACTION_READ_UNCOMMITTED);
	}
	if (isoLevelName.equals("TRANSACTION_READ_COMMITTED")) {
	    return(ControlDescriptor.TRANSACTION_READ_COMMITTED);
	}
	if (isoLevelName.equals("TRANSACTION_REPEATABLE_READ")) {
	    return(ControlDescriptor.TRANSACTION_REPEATABLE_READ);
	}
	if (isoLevelName.equals("TRANSACTION_SERIALIZABLE")) {
	    return(ControlDescriptor.TRANSACTION_SERIALIZABLE);
	}

	return(TX_ISOLEVEL_NOTDEFINED);
    }

    /**
     * Gets the string value of a transaction isolation level from a transaction isolation
     * level int.
     * Used for tracing purpose.
     *
     * @param  int    isoLevel The Transaction Isolation Leval
     * @return String          The Transaction Isolation Level Name
     */
    public static String getTransactionIsolationLevelName (int isoLevel) {

	  switch (isoLevel) {
	  case TX_ISOLEVEL_NOTDEFINED:
	      return("TX_ISOLEVEL_NOTDEFINED");
	  case ControlDescriptor.TRANSACTION_READ_UNCOMMITTED:
	      return("TRANSACTION_READ_UNCOMMITTED");
	  case ControlDescriptor.TRANSACTION_READ_COMMITTED:
	      return("TRANSACTION_READ_COMMITTED");
	  case ControlDescriptor.TRANSACTION_REPEATABLE_READ:
	      return("TRANSACTION_REPEATABLE_READ");
	  case ControlDescriptor.TRANSACTION_SERIALIZABLE:
	      return("TRANSACTION_SERIALIZABLE");
	  default:
	      return("TRANSACTION_ISOLEVEL_NOTDEFINED");
	  }
      }

    /**
     * Gets the int value of a runAsMode security attribute from a runAsMode string.
     * Used for tracing purpose.
     *
     * @param  String runAsModeName  The Transaction Isolation Level Name
     * @return int                   The Transaction Isolation Level
     */
    public static int getRunAsMode (String runAsModeName) {

	if (runAsModeName.equals("CLIENT_IDENTITY")) {
	    return(ControlDescriptor.CLIENT_IDENTITY);
	}
	if (runAsModeName.equals("SPECIFIED_IDENTITY")) {
	    return(ControlDescriptor.SPECIFIED_IDENTITY);
	}
	if (runAsModeName.equals("SYSTEM_IDENTITY")) {
	    return(ControlDescriptor.SYSTEM_IDENTITY);
	}

	return(RUNASMODE_NOTDEFINED);
    }

    /**
     * Gets the string value of a runAsMode security attribute from a runAsMode security attribute
     * int.
     * Used for tracing purpose.
     *
     * @param  int    runAsMode The runAsMode security attribute
     * @return String           The runAsMode security attribute Name
     */
    public static String getRunAsModeName (int runAsMode) {

	  switch (runAsMode) {
	  case RUNASMODE_NOTDEFINED:
	      return("RUNASMODE_NOTDEFINED");
	  case ControlDescriptor.CLIENT_IDENTITY:
	      return("CLIENT_IDENTITY");
	  case ControlDescriptor.SPECIFIED_IDENTITY:
	      return("SPECIFIED_IDENTITY");
	  case ControlDescriptor.SYSTEM_IDENTITY:
	      return("SYSTEM_IDENTITY");
	  default:
	      return("RUNASMODE_NOTDEFINED");
	  }
      }

    /**
     * Gets the int value of a Session State Management Type from a string.
     * Used for tracing purpose.
     *
     * @param  String stateTypeName  The Session State Management Type Name
     * @return int                   The Session State Management Type
     */
    public static int getSessionStateType (String sessionStateTypeName) {

	if (sessionStateTypeName.equals("STATEFUL_SESSION")) {
	    return(SessionDescriptor.STATEFUL_SESSION);
	}
	if (sessionStateTypeName.equals("STATELESS_SESSION")) {
	    return(SessionDescriptor.STATELESS_SESSION);
	}

	return(SESSIONSTATETYPE_NOTDEFINED);
    }

    /**
     * Gets the string value of a Session State Management Type from a int value.
     * Used for tracing purpose.
     *
     * @param  int    sessionStateType The Session State Management Type
     * @return String                  The Session State Management Type Name
     */
    public static String getSessionStateTypeName (int sessionStateType) {

	  switch (sessionStateType) {
	  case SESSIONSTATETYPE_NOTDEFINED:
	      return("SESSIONSTATETYPE_NOTDEFINED");
	  case SessionDescriptor.STATEFUL_SESSION:
	      return("STATEFUL_SESSION");
	  case SessionDescriptor.STATELESS_SESSION:
	      return("STATELESS_SESSION");
	  default:
	      return("SESSIONSTATETYPE_NOTDEFINED");
	  }
      }
}
