/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.tools;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectInputStream;
import java.io.PrintWriter;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.Properties;
import javax.ejb.deployment.AccessControlEntry;
import javax.ejb.deployment.ControlDescriptor;
import javax.ejb.deployment.DeploymentDescriptor;
import javax.ejb.deployment.EntityDescriptor;
import javax.ejb.deployment.SessionDescriptor;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.lib.JavaType;
import org.objectweb.jonas.lib.Util;


/**
 *  This class allows to disassembles a given serialized deployment descriptor.
 */

public class DumpDD {

    static final int trace = Trace.DB_19;

    private EntityDescriptor  edesc;
    private SessionDescriptor sdesc;
    private String fTxtName = null;
    private PrintWriter fTxt = null;
    private String fPropsName = null;
    private PrintWriter fProps = null;
    boolean isProps = false;

    public DumpDD(String fileSerName, String fileTxtName)
	throws IOException, ClassNotFoundException {

	try {
	    ObjectInput in1 = new ObjectInputStream (new FileInputStream(fileSerName)); 
	    sdesc = (SessionDescriptor)in1.readObject();
	    edesc = null;
	} catch (ClassCastException e) {
	    ObjectInput in2 = new ObjectInputStream (new FileInputStream(fileSerName)); 
	    edesc = (EntityDescriptor)in2.readObject();
	    sdesc = null;
	}

	fTxtName = new String(fileTxtName);
	fPropsName = null;
	int i = fileTxtName.lastIndexOf(".");
	if (i == -1) {
	    fPropsName = new String(fileTxtName + ".properties");
	} else {
	    fPropsName = new String(fileTxtName.substring(0, i) + ".properties");
	}
	fTxt = new PrintWriter(new FileWriter(fTxtName));
	fProps = new PrintWriter(new FileWriter(fPropsName));
    }

    public void print() {
	if (edesc != null) {
	    printED(edesc);
	}
	if (sdesc != null) {
	    printSD(sdesc);
	}
	fTxt.close();
	fProps.close();
	Trace.outln(trace, "DumpDD: The given Deployment Descriptor is successfully dumped in");
	Trace.outln(trace, "        "+fTxtName);
	if (isProps) {
	    Trace.outln(trace, "        "+fPropsName);
	}
    }

    private void printED(EntityDescriptor ed) {

	fTxt.println("");
	fTxt.println("EntityDescriptor {");

	printDD(ed);

	Field [] cmfs = ed.getContainerManagedFields();
	if (cmfs != null) {
	    fTxt.println("    ContainerManagedFields   =");
	    fTxt.println("      {");
	    for (int i=0; i< cmfs.length; i++) {
		fTxt.println("       	 "+cmfs[i].getName()+";");
	    }
	    fTxt.println("      };");
	}

	fTxt.println("    PrimaryKeyClassName      = "+ed.getPrimaryKeyClassName()+";");
	fTxt.println("    Reentrant                = "+ed.getReentrant()+";");

	fTxt.println("}");
	fTxt.println("");

    }

    private void printSD(SessionDescriptor sd) {

	fTxt.println("");
	fTxt.println("SessionDescriptor {");
	printDD(sd);
	fTxt.println("    SessionTimeout           = "+sd.getSessionTimeout()+";");
	fTxt.println("    StateManagementType      = "+Util.getSessionStateTypeName(sd.getStateManagementType())+";");
	fTxt.println("}");
	fTxt.println("");

    }

    private void printDD(DeploymentDescriptor dd) {

	fTxt.println("    BeanHomeName             = \""+dd.getBeanHomeName()+"\";");
	fTxt.println("    EnterpriseBeanClassName  = "+dd.getEnterpriseBeanClassName()+";");
	fTxt.println("    HomeInterfaceClassName   = "+dd.getHomeInterfaceClassName()+";");
	fTxt.println("    RemoteInterfaceClassName = "+dd.getRemoteInterfaceClassName()+";");

	//AccessControlEntry [] aces = dd.getAccessControlEntries();
	//if (aces != null) {
	//    fTxt.println("    AccessControlEntries     = ");
	//    fTxt.println("      {");
	//    fTxt.println("       	 ?????");
	//    fTxt.println("      };");
	//}

	ControlDescriptor [] cds = dd.getControlDescriptors();
	if (cds != null) {
	    fTxt.println("    ControlDescriptors       =");
	    fTxt.println("      {");
	    for (int i = 0; i < cds.length; i++) {
		String isoLevel   = Util.getTransactionIsolationLevelName(cds[i].getIsolationLevel());
		String methodName = null;
		if (cds[i].getMethod() != null) {
		    Method method = cds[i].getMethod();
		    methodName = new String(method.getName()+"(");
		    Class[] params = method.getParameterTypes();
		    for (int j=0; j<params.length; j++) {
			if (j>0) {
			    methodName = methodName.concat(", ");
			}
			methodName = methodName.concat(JavaType.getName(params[j]));
		    }
		    methodName = methodName.concat(")");
		}
		String runAsMode = Util.getRunAsModeName(cds[i].getRunAsMode());
		String txAttr    = Util.getTransactionAttributeName(cds[i].getTransactionAttribute());
		fTxt.println("        {");
		fTxt.println("          IsolationLevel       = "+isoLevel+";");
		if (cds[i].getMethod() != null) {
		    fTxt.println("          Method               = "+methodName+";");
		}
		//fTxt.println("          RunAsIdentity        = ?????");
		fTxt.println("          RunAsMode            = "+runAsMode+";");
		fTxt.println("          TransactionAttribute = "+txAttr+";");
		fTxt.println("        };");
	    }
	    fTxt.println("      };");
	}

	Properties ps = dd.getEnvironmentProperties();
	if (ps != null) {
	    isProps = true;
	    fTxt.println("    EnvironmentProperties    = \""+fPropsName+"\";");
	    for (Enumeration enum = ps.propertyNames(); enum.hasMoreElements(); ) {
		String attr  = (String)enum.nextElement();
		String value = ps.getProperty(attr);
		fProps.println(attr+"="+value);
	    }
	}
    }

    public static void main(String args[]) {
	
	boolean isHelp         = false;
	boolean isVerbose      = false;
	String  fileInputName  = null;
	String  fileOutputName = null;
	String  dirOutputName  = null;
	String  fileTxtName;

	// Get args
	for (int argn=0; argn<args.length; argn++) {
	    String arg = args[argn];
	    if (arg.equals("-help") || arg.equals("-?")) {
		isHelp = true;
		continue;
	    }
	    if (arg.equals("-verbose")) {
		isVerbose = true;
		continue;
	    }
	    if ((argn<args.length) && arg.equals("-d")) {
		dirOutputName = args[++argn];
		continue;
	    }
	    if ((argn<args.length) && arg.equals("-f")) {
		fileOutputName = args[++argn];
		continue;
	    }
	    fileInputName = args[argn];
	}

	// Trace configure
	if (isVerbose) {
	    Trace.configure("out", "err", trace, false);
	} else {
	    Trace.configure("out", "err", Trace.DB_NONE, false);
	}


	// Usage ?
	if (isHelp) {
	    usage();
	    System.exit(0);
	}

	// Check args
	if (fileInputName == null) {
	    usage();
	    System.exit(2);
	}

	// Init the name of the generated file
	if (dirOutputName == null) {
	    fileTxtName = new String("");
	} else {
	    fileTxtName = new String(dirOutputName + File.separatorChar);
	}
	if (fileOutputName == null) {
	    int i = fileInputName.lastIndexOf(".");
	    if (i == -1) {
		fileOutputName = new String(fileInputName);
	    } else {
		fileOutputName = new String(fileInputName.substring(0, i) + ".txt");
	    }
	}
	fileTxtName = fileTxtName.concat(fileOutputName);
	
	// Dump the content of the .ser file
	try {
	    DumpDD ddd = new DumpDD(fileInputName, fileTxtName);
	    ddd.print();
	} catch (Exception e) {
	    Trace.errln("DumpDD ERROR:  " + e.toString());
	    System.exit(2);
	}
	// End
	System.exit(0);
    }

    private static void usage() {
	Trace.outln("Usage: java org.objectweb.jonas.tools.DumpDD -help");
	Trace.outln("         to print this help message");
	Trace.outln("");
	Trace.outln("or     java org.objectweb.jonas.tools.DumpDD <Options> <Input_File>");
	Trace.outln("         to dump an Enterprise Bean's serialized deployment descriptor");
	Trace.outln("           Options include:"); 
	Trace.outln("             -d <output_dir>  specify where to place the generated file");
	Trace.outln("             -f <output_file> specify the name of the generated file");
	Trace.outln("             -verbose");
	Trace.outln("           Input_File  file name of the serialized deployment descriptor");
	Trace.outln("                       to dump.");
    }
}

