/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.tools;

import java.lang.String;
import java.lang.reflect.Field;
import org.objectweb.jonas.lib.DD;
import org.objectweb.jonas.lib.JavaType;
import org.objectweb.jonas.common.Trace;


/**
 *  This class allows to generate the source of the class that implements
 *  the Enterprise bean's remote interface of a given Entity Enterprise Java Bean
 *  with container managed persistence.
 *  Its extends GenICEntityRemote class, itself extends GenICRemote class.
 */

abstract class GenICEntityCMRemote extends GenICEntityRemote {

    // List of the container-managed fields of the bean
    Field[] fieldsPers     = null;
    // List of the fields of the PrimaryKey
    Field[] fieldsPK       = null;
    // List of the container-managed fields of the bean which are not PrimaryKey's fields
    Field[] fieldsPersNoPK = null;

    /**
     * GenICEntityCMRemote Constructor
     *
     * @param dd       deployment descriptor of the Entity Bean with container managed persistence
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICEntityCMRemote(DD dd, String fileName) throws GenICException {

	super(dd, fileName);

	// Get the list of the container-managed fields
	// and check if all fields can be treated.
	fieldsPers = (ddesc.getEntityDescriptor()).getContainerManagedFields();
	for (int i = 0; i<fieldsPers.length; i++) {
	    if ((JavaType.getSQLGetMethod(fieldsPers[i].getType()) == null) &&
		(JavaType.getSQLSetMethod(fieldsPers[i].getType()) == null)) {
		throw new GenICException("Cannot container manage the bean's field '" +
					 fieldsPers[i].getName() +
					 "' (Type '" + JavaType.getName(fieldsPers[i].getType()) + "')");
	    }
	}

	// Get the list of the PrimaryKey fields
	// and check if all fields can be treated
	try {
	    fieldsPK = ddesc.getFieldsOfPrimaryKey();
	} catch (Exception e) {
	    throw new GenICException(e.getMessage());
	}
	for (int i = 0; i<fieldsPK.length; i++) {
	    if ((JavaType.getSQLGetMethod(fieldsPK[i].getType()) == null) &&
		(JavaType.getSQLSetMethod(fieldsPK[i].getType()) == null)) {
		throw new GenICException("Cannot container manage the primary key's field '" +
					 fieldsPK[i].getName() +
					 "' (Type '" + JavaType.getName(fieldsPK[i].getType()) + "')");
	    }
	}

	// Check if The names of the fields in the Primary Key class is a subset
	// of the names of the container-managed fields of the Entity Bean
	for (int i=0; i<fieldsPK.length; i++) {
	    boolean isFieldContainerManaged = false;
	    String fieldPKName = fieldsPK[i].getName();
	    for (int j=0; (j<fieldsPers.length && !isFieldContainerManaged); j++) {
		if (fieldPKName.equals(fieldsPers[j].getName())) {
		    isFieldContainerManaged = true;
		}
	    }
	    if (!isFieldContainerManaged) {
		throw new GenICException("The field '" + fieldsPK[i].getName() +
					 "' of the Primary Key class is not a " +
					 "container-managed field of the Entity Bean");
	    }
	}

	// Build the list of the container-managed fields
	// which are not Primary Key's fields
	fieldsPersNoPK = new Field[fieldsPers.length-fieldsPK.length];
	int l = 0;
	for (int i=0; i<fieldsPers.length; i++) {
	    boolean found=false;
	    for (int j=0; (j<fieldsPK.length && !found); j++) {
		if ((fieldsPers[i].getName()).equals(fieldsPK[j].getName())) {
		    found = true;
		}
	    }
	    if (!found) {
		fieldsPersNoPK[l] = fieldsPers[i];
		l++;
	    }
	}
    }

    /**
     * Source generation of the import clauses
     * @exception GenICException in error case
     */
    abstract void genImport() throws GenICException;

    /**
     * Source generation of the isModifiedData() method.
     * The source generation of the createData(), removeData(), loadData(), storeData() methods
     * is done in the Sub-class.
     * @exception GenICException in error case
     */
    void genAccessData() throws GenICException {
	src.println("public boolean isModifiedData(EntityBean bean) {");
	src.indentPlus();
	String isModifiedMethodName = ddesc.getIsModifiedMethodName();
	if (isModifiedMethodName != null) {
	    src.println("boolean b = (("+ejbBeanName+")bean)."+isModifiedMethodName+"();");

	} else {
	    src.println("boolean b = true;");
	}
	src.println("Trace.outln(Trace.DB_15, \"" + wrpRemoteBaseName + ".isModifiedData(EntityBean) --> \" + b);");
	src.println("return(b);");
	src.indentMinus();
	src.println("}");
	src.println();
    }

}

