/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.tools;


import java.lang.String;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.lib.DD;


/**
 *  This class allows to generate the source of the class that implements
 *  the Handle of a given Entity Enterprise Java Bean.
 *  Its extends GenICHandle class.
 */

class GenICEntityHandle extends GenICHandle {

    // Full names of the interfaces and the classes
    String ejbPrimaryKeyName = null;

    /**
     * GenICEntityHandle Constructor
     *
     * @param dd       deployment descriptor of the Entity Enterprise Java Bean
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICEntityHandle (DD dd, String fileName) throws GenICException {
	super(dd, fileName);

	ejbPrimaryKeyName = (ddesc.getEntityDescriptor()).getPrimaryKeyClassName();
	if ((ejbPrimaryKeyName == null) || (ejbPrimaryKeyName.length() < 1)) {
	    throw new GenICException("The PrimaryKeyClassName MUST BE defined "+
				     "in the EntityDescriptor");
	}

    }

    /**
     * Name of the superclass of the generated class
     */
    String getSuperClass() {
	return("EntityHandleImpl");
    }

    /**
     * Source generation of the import clauses
     * @exception GenICException in error case
     */
    void genImport() throws GenICException {
	src.println("import java.lang.String;");
	src.println("import java.rmi.RemoteException;");
	src.println("import java.util.Properties;");
	src.println("import javax.ejb.EJBObject;");
	src.println("import javax.ejb.HomeHandle;");
	src.println();
	src.println("import org.objectweb.jonas.container.JEntityHome;");
	src.println("import org.objectweb.jonas.container.EntityHandleImpl;");
    }

    /**
     * Source generation of the attributes part
     * @exception GenICException in error case
     */
    void genAttributes() throws GenICException {
    }

    /**
     * Source generation of the constructor method
     * @exception GenICException in error case
     */
    void genConstructor() throws GenICException {

	src.println("public " + wrpHandleBaseName + "("+ wrpRemoteName +
		    " ejbobject) {");

	src.indentPlus();
	src.println("super(ejbobject);");
	//src.println("Trace.outln(Trace.DB_15, \"" + wrpHandleBaseName + "(ejbobject)\");");
	src.println("try {");
	src.indentPlus();
	src.println("pk = ("+ejbPrimaryKeyName+")ejbobject.getPrimaryKey();");
	src.indentMinus();
	src.println("} catch (Exception e) {");
	src.indentPlus();
	src.println("pk = null;");
	src.indentMinus();
	src.println("}");  // try-catch
	src.indentMinus();
	src.println("}");
    }

    /**
     * Source generation of the getEJBObject() method
     * @exception GenICException in error case
     */
    void genGetEJBObject() throws GenICException {
	src.println("public EJBObject getEJBObject() throws RemoteException {");
	src.indentPlus();
	src.println(ejbRemoteName + " ejbobj = null;");
	src.println("try {");
	src.indentPlus();
	src.println(ejbHomeName + " h = ("+ejbHomeName+")homehandle.getEJBHome();");
	src.println("ejbobj = h.findByPrimaryKey(("+ejbPrimaryKeyName+")pk);");
	src.indentMinus();
	src.println("} catch (Exception e) {");
	src.indentPlus();
	src.println("throw new RemoteException(\""+wrpHandleBaseName+".getEJBObject(): \"+e, e);");
	src.indentMinus();
	src.println("}");  // try-catch
	src.println("return(ejbobj);");
	src.indentMinus();
	src.println("}");
    }

}

