/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.tools;


import java.lang.String;
import java.lang.reflect.Method;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.lib.DD;


/**
 *  This class allows to generate the source of the class that implements
 *  the Enterprise bean's Home interface of a given Session Enterprise Java Bean.
 *  Its extends GenICHome class.
 */

class GenICSessionHome extends GenICHome {

    /**
     * GenICSessionHome Constructor
     *
     * @param dd       deployment descriptor of the Sesion Enterprise Java Bean
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICSessionHome(DD dd, String fileName) throws GenICException {
	super(dd, fileName);
    }

    /**
     * Name of the superclass of the generated class
     */
    String getSuperClass() {
	return(new String("JSessionHome"));
    }

    /**
     * Source generation of the import clauses
     * @exception GenICException in error case
     */
    void genImport() throws GenICException {
	src.println("import java.lang.Error;");
	src.println("import java.lang.Object;");
	src.println("import java.lang.RuntimeException;");
	src.println("import java.rmi.RemoteException;");
	src.println("import javax.ejb.deployment.SessionDescriptor;");
	src.println("import org.objectweb.jonas.container.JSessionHome;");
	src.println("import org.objectweb.jonas.container.SessionContextImpl;");
	src.println("import org.objectweb.jonas.common.Trace;");
    }

    /**
     * Source generation of the constructor method
     * @exception GenICException in error case
     */
    void genConstructor() throws GenICException {
	src.println("public " + wrpHomeBaseName +
		    "(SessionDescriptor desc, String BeanName) throws RemoteException {");
	src.indentPlus();
	src.println("super(desc, BeanName);");
	src.println("Trace.outln(Trace.DB_15, \"" + wrpHomeBaseName + "(\"+BeanName+\")\");");
	src.indentMinus();
	src.println("}");
    }

    /**
     * Source generation for the given create() method described in the Home interface
     * @exception GenICException in error case
     */
    void genCreate(Method method) throws GenICException {

	Class paramsTable[] = method.getParameterTypes();

	genHeaderMethod(method);
	src.println(wrpRemoteBaseName + " ejbobj = null;");
	src.println("ejbobj = new " + wrpRemoteBaseName + "(this);");
	src.println(ejbBeanName + " eb = null;");
	src.println("SessionContextImpl ctx = null;");
	src.println("try {");
	src.indentPlus();
	src.println("ctx = ejbobj.createContext();");
	src.println("eb = ("+ejbBeanName +")ctx.getInstance();");
	src.indentMinus();
	src.println("} catch (Exception e) {");
	src.indentPlus();
	src.println("Trace.errln(\"Failed to create a new instance of the bean class "+ejbBeanName+"\");");
	src.println("Trace.errln(e);");
	src.println("throw new RemoteException(\"Failed to create a new instance of the bean class "+ejbBeanName+"\", e);");
	src.indentMinus();
	src.println("}");
	src.println("try {");
	src.indentPlus();
	src.print("eb.ejbCreate(");
	for (int p=1; p <= paramsTable.length; p++) {
	    src.print("p" + p, false);
	    if (p < paramsTable.length) {
		src.print(", ", false);
	    }
	}
	src.println(");", false);
	genCatchBeanException(method, "ejbCreate");
	src.indentMinus();
	src.println("}");
	src.println("return(ejbobj);");
	src.indentMinus();
	src.println("}");
    }

    /**
     * Source generation for the given findByPrimaryKey() method described in the Home interface
     * @exception GenICException in error case
     */
    void genFindByPK(Method method) throws GenICException {
	// Nothing to do !!!
	// No findByPrimaryKey() method in case of Session
    }

    /**
     * Source generation for the given findXXX() method described in the Home interface
     * @exception GenICException in error case
     */
    void genFindOther(Method method) throws GenICException {
	// Nothing to do !!!
	// No findXXX() methods in case of Session
    }

}

